<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<!--
    transform Generic into PostgreSQL
-->
<xsl:output method="text"/>
  <xsl:template match="/datadesigner">
    <xsl:for-each select="domains/domain">
      <xsl:call-template name="doDomain"/>
    </xsl:for-each>
    <xsl:for-each select="sequences/sequence">
      <xsl:call-template name="doSequence"/>
    </xsl:for-each>
    <xsl:for-each select="entities/entity">
      <xsl:call-template name="doEntity"/>
    </xsl:for-each>
    <xsl:for-each select="views/view">
      <xsl:call-template name="doView"/>
    </xsl:for-each>
    <xsl:for-each select="procedures/procedure">
      <xsl:call-template name="doProcedure"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="doSequence">
<xsl:text>
CREATE SEQUENCE </xsl:text><xsl:value-of select="@pname"/>
<xsl:if test="increment != &quot;&quot;"> INCREMENT <xsl:value-of select="increment"/></xsl:if>
<xsl:if test="minvalue != &quot;&quot;"> MINVALUE <xsl:value-of select="minvalue"/></xsl:if>
<xsl:if test="maxvalue != &quot;&quot;"> MAXVALUE <xsl:value-of select="maxvalue"/></xsl:if>
<xsl:if test="start != &quot;&quot;"> START <xsl:value-of select="start"/></xsl:if>
<xsl:if test="cache != &quot;&quot;"> CACHE <xsl:value-of select="cache"/></xsl:if>
<xsl:if test="cycle = &quot;true&quot;"> CYCLE</xsl:if>
<xsl:text>;
</xsl:text>
  </xsl:template>

  <xsl:template name="doDomain">
<xsl:text>
CREATE DOMAIN </xsl:text><xsl:value-of select="@pname"/><xsl:text> AS </xsl:text><xsl:value-of select="basetype"/>
<xsl:if test="length &gt; 0"><xsl:text>(</xsl:text><xsl:value-of select="length"/>
<xsl:if test="decimals &gt; 0"><xsl:text>,</xsl:text><xsl:value-of select="decimals"/></xsl:if><xsl:text>)</xsl:text></xsl:if>
<xsl:if test="nullable = &quot;false&quot;"> NOT NULL</xsl:if>
<xsl:if test="default != &quot;&quot;"> DEFAULT <xsl:value-of select="default"/> </xsl:if>
<xsl:if test="check != &quot;&quot;"> CHECK <xsl:value-of select="check"/> </xsl:if>
<xsl:text>;
</xsl:text>
  </xsl:template>

  <xsl:template name="doEntity">
    <xsl:text>
CREATE TABLE </xsl:text><xsl:value-of select="@pname"/>
    <xsl:text> (
</xsl:text>
    <xsl:call-template name="doAttributes"><xsl:with-param name="entity"><xsl:value-of select="@pname"/></xsl:with-param></xsl:call-template>
    <xsl:call-template name="doConstraints"/>
    <xsl:text>
);
</xsl:text>
    <xsl:call-template name="doIndex"><xsl:with-param name="entity"><xsl:value-of select="@pname"/></xsl:with-param></xsl:call-template>
    <xsl:call-template name="doTrigger"><xsl:with-param name="entity"><xsl:value-of select="@pname"/></xsl:with-param></xsl:call-template>
  </xsl:template>

  <xsl:template name="doAttributes">
    <xsl:param name="entity"/>
    <xsl:for-each select="attributes/attribute">
      <xsl:text>    </xsl:text><xsl:value-of select="@pname"/>
      <xsl:text> </xsl:text><xsl:value-of select="type"/>
      <xsl:if test="length &gt; 0"><xsl:text>(</xsl:text><xsl:value-of select="length"/>
      <xsl:if test="decimals &gt; 0"><xsl:text>,</xsl:text><xsl:value-of select="decimals"/></xsl:if><xsl:text>)</xsl:text></xsl:if>
      <xsl:if test="nullable = &quot;false&quot;"> NOT NULL</xsl:if>
      <xsl:if test="default != &quot;&quot;"> DEFAULT <xsl:value-of select="default"/> </xsl:if>
      <xsl:if test="check != &quot;&quot;"> CHECK (<xsl:value-of select="check"/>)</xsl:if>
      <xsl:if test="generated = &quot;true&quot;"><xsl:text> DEFAULT nextval('</xsl:text><xsl:value-of select="$entity"/><xsl:text>_seq')</xsl:text></xsl:if>
      <xsl:if test="not (position() = last())"><xsl:text>,
</xsl:text></xsl:if>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="doConstraints">
    <xsl:for-each select="constraints/constraint">
      <xsl:text>,
    </xsl:text>
      <xsl:choose>
        <xsl:when test="type = &quot;primarykey&quot;">
	  <xsl:text>PRIMARY KEY</xsl:text><xsl:call-template name="doSimpleAttrs"/>
	</xsl:when>
        <xsl:when test="type = &quot;foreignkey&quot;">
	  <xsl:text>FOREIGN KEY</xsl:text><xsl:call-template name="doSimpleAttrs"/>
	  <xsl:text> REFERENCES </xsl:text>
	  <xsl:variable name="rel" select="relation"/>
	  <xsl:for-each select="../../../../relations/relation">
	    <xsl:if test="$rel = @name">
	      <xsl:value-of select="parent"/>
	      <xsl:variable name="par" select="parent"/>
	      <xsl:for-each select="../../entities/entity">
	        <xsl:if test="$par = @name">
		  <xsl:for-each select="constraints/constraint">
		    <xsl:if test="type = &quot;primarykey&quot;"><xsl:call-template name="doSimpleAttrs"/></xsl:if>
		  </xsl:for-each>
		</xsl:if>
	      </xsl:for-each>
	    </xsl:if>
	  </xsl:for-each>
	</xsl:when>
        <xsl:when test="type = &quot;unique&quot;">
	  <xsl:text>UNIQUE</xsl:text><xsl:call-template name="doSimpleAttrs"/>
	</xsl:when>
      </xsl:choose>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template name="doSimpleAttrs">
    <xsl:text> ( </xsl:text>
    <xsl:for-each select="simpleattributes/simpleattribute">
      <xsl:value-of select="@name"/>
      <xsl:if test="not (position() = last())"><xsl:text>, </xsl:text></xsl:if>
    </xsl:for-each>
    <xsl:text> )</xsl:text>
  </xsl:template>

  <xsl:template name="doView">
<xsl:text>
CREATE VIEW </xsl:text><xsl:value-of select="@pname"/>;
  </xsl:template>

  <xsl:template name="doIndex">
    <xsl:param name="entity"/>
    <xsl:for-each select="indexes/index">
<xsl:text>
CREATE </xsl:text><xsl:if test="unique = &quot;true&quot;">UNIQUE </xsl:if>INDEX <xsl:value-of select="@pname"/> ON <xsl:value-of select="$entity"/>
<xsl:call-template name="doSimpleAttrs"/>
<xsl:text>;
</xsl:text>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="doTrigger">
    <xsl:param name="entity"/>
    <xsl:for-each select="triggers/trigger">
<xsl:text>
CREATE TRIGGER </xsl:text><xsl:value-of select="@pname"/><xsl:text> </xsl:text>
    <xsl:value-of select="time"/><xsl:text> </xsl:text>
    <xsl:for-each select="event">
      <xsl:value-of select="."/>
      <xsl:if test="not (position() = last())"><xsl:text> OR </xsl:text></xsl:if>
    </xsl:for-each>
    <xsl:text> ON </xsl:text><xsl:value-of select="$entity"/>
    <xsl:text> FOR EACH </xsl:text>
    <xsl:value-of select="type"/>
    <xsl:text> EXECUTE PROCEDURE </xsl:text><xsl:value-of select="procedure"/>
<xsl:text>;
</xsl:text>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="doProcedure">
<!-- NOT SUPPORTED YET
CREATE FUNCTION <xsl:value-of select="@pname"/>;
-->
  </xsl:template>

</xsl:stylesheet>
